#include <SDL.h>
#include <SDL_ttf.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

void help(char* prog) {
    printf("usage: %s [-i] [-b] ptsize xsize ysize xdots ydots inttf outbmp\n", prog);
    printf("\t-i: italic\n");
    printf("\t-b: bold\n");
    printf("example: %s 16 256 128 16 16 kochi.ttf kochi.bmp #(NeHe17 style)\n");
    exit(0);
}

int main(int argc, char* argv[]) {
    int style;
    int ptsize;
    int xsize, ysize;
    int xdots, ydots;
    int xchars, ychars;
    char* inttf;
    char* outbmp;
    char* prog;
    int ai;
    int i;

    SDL_Surface* surf;
    TTF_Font* font;

    prog = argv[0];

    if (argc < 7) {
        help(prog);
    }

    ai = 1;
    style = 0;
    while (1) {
        if (strcmp("-i", argv[ai]) == 0) {
            style |= TTF_STYLE_ITALIC;
            ai++;
        }
        else if (strcmp("-b", argv[ai]) == 0) {
            style |= TTF_STYLE_BOLD;
            ai++;
        }
        else {
            break;
        }
    }

    if (argc-ai != 7) {
        help(prog);
    }

    ptsize = atoi(argv[ai++]);
    xsize = atoi(argv[ai++]);
    ysize = atoi(argv[ai++]);
    xdots = atoi(argv[ai++]);
    ydots = atoi(argv[ai++]);
    inttf = argv[ai++];
    outbmp = argv[ai++];

    xchars = xsize / xdots;
    ychars = ysize / ydots;

    if (!ptsize || !xdots || !ydots) {
        help(prog);
    }

    SDL_Init(SDL_INIT_VIDEO | SDL_INIT_NOPARACHUTE);
    TTF_Init();

    surf = SDL_CreateRGBSurface(SDL_SWSURFACE, xsize, ysize, 24, 0,0,0,0);
    font = TTF_OpenFont(inttf, ptsize);
    TTF_SetFontStyle(font, style);

    for (i = 0; i < 96; i++) {
        char buf[2];
        SDL_Surface* t;
        SDL_Color col;
        SDL_Rect d;

        col.r = 255;
        col.g = 255;
        col.b = 255;

        buf[0] = i+32;
        buf[1] = '\0';

        t = TTF_RenderText_Blended(font, buf, col);

        d.x = (i%xchars)*xdots + (xdots-t->w)/2;
        d.y = (i/xchars)*ydots + (ydots-t->h)/2;
        SDL_BlitSurface(t, NULL, surf, &d);

        SDL_FreeSurface(t);
    }

    SDL_SaveBMP(surf, outbmp);

    TTF_Quit();
    SDL_Quit();
}
